/*
 * You must include  io.h  first
 * TTY Options
 */

#define ECHO			0x01
#define EDIT			0x02
#define ETAB			0x04
#define ERS				0x08
#define EDEL			0x10
#define IGATE			0x20
#define XON_XOFF		0x40
#define OFLOW			0x40
#define MAPCR			0x80
#define OUTPUT_PAGED	0x100
#define IFLOW			0x200
#define HFLOW			0x800
#define EXPAND_FUNCTION	0x1000
#define COMM_ERRORS		0x2000

/*
 * TTY flags
 */

#define BUSY_WAIT		0x00
#define INT_DRIVEN		0x01
#define INT_FUNC		0x02
#define FUNC_CALL		0x03
#define TX_RDY			0x04
#define TTY_BUSY		0x08
#define BREAK_BIT		0x10
#define FUN_FLAG		0x20
#define INS_FLAG		0x40
#define RE_ECHO			0x80
#define BREAK_PENDING	0x100
#define ESCAPE_PENDING	0x200
#define WAITING_KEY		0x400
#define KEY_FOUND		0x800
#define PRT_BUSY		0x1000
#define HANGUP_PENDING	0x2000
#define FLOW_CONTROLLED	0x4000
#define PRT_READY		0x8000

/*
 * TTY EXTENDED FLAGS
 */

#define ECHO_PENDING	0x01
#define TRAIL_BACKSPACE	0x02
#define WINDOW_SWITCH	0x04
#define PRTSC_PENDING	0x08
#define FUNCTION_KEY	0x10
#define PRECEDE_DEL		0x20
#define COMMERR_PENDING	0x40
#define POLL_PRINTER	0x4000
#define HANGUP			0x4000
#define DISALLOW_COLOUR	0x4000
#define LOCK_DEVICE		0x8000

#define PROMPT_MSG		0x0002

#define NUL		0x00
#define ETX		0x03
#define EOT		0x04
#define BS		0x08
#define HT		0x09
#define LF		0x0a
#define FF		0x0c
#define CR		0x0d
#define DC1		0x11
#define DC3		0x13
#define CAN		0x18
#define RS		0x1e
#define SP		0x20
#define DEL		0x7f

#define STATE	0
#define CHAR	1

#define HT_STATE	1
#define RS_STATE	8
#define DEL_STATE	5



struct dev_entry {
	char dev_name[6];
	unsigned tty_options;
	char tty_page;
	char tty_type;
	char tty_BREAK;
	char tty_ESCAPE;
	char tty_RUBOUT;
	char tty_CANCEL;
	char tty_EOT;
	char tty_UP;
	char tty_RIGHT;
	char tty_DOWN;
	char tty_LEFT;
	char tty_HOME;
	char tty_CLEAR;
	char tty_FUNCTION;
	char tty_INSERT;
	char tty_DELETE;
	unsigned tty_ioport;
	unsigned tty_ent_spare[2];
	char tty_dummy;
	char tty_bs_count;
	unsigned tty_eflags;
	unsigned tty_flags;
	char tty_row;
	char tty_col;
	char tti_echo;
	char tti_ecnt;
	char tti_eptr;
	char tti_in;
	char tti_out;
	char tti_bol;
	char tti_eol;
	char tti_col;
	char tto_index;
	char tto_length;
	char tty_state;
	char tty_dev_no;
	char *tti_bufp;
	char *tto_bufp;
	unsigned dev_baud;
	char dev_parity;
	char dev_stop_bits;
	char dev_data_bits;
	char tti_echar;
	unsigned tto_dr;			/* also tto_function */
	char dev_entry_spare2[2];
	} ;

struct window_dev_entry {
	char window_dev_entry_filler[54];
	unsigned window_buf_seg;
	unsigned window_ctrl_block;
	char window_dev_entry_filler2[6];
	} ;

#define ATTR_SIZE 26

struct attr_msg {
	char mtype;
	char devno;
	struct dev_entry tty_ent;
	} ;

struct state_entry {
	char state_cursor_x;
	char state_cursor_y;
	char state_spare1;
	char state_spare2;
	char state_spare3;
	char state_spare4;
	char state_cursor_type;
	char state_attr;
	char state_colour;
	char state_fill_colour;
	char state_screen_width;
	char state_screen_height;
	char state_spares[4];
	} ;

struct prompt_msg {
	char mtype;
	char new_disk;
	char on_tty;
	} ;

struct window_msg {
	char mtype;
	char wdw_devno;
	unsigned wdw_segment;
	char wdw_name[6];
	} ;


#define AWAIT_FILL	1
#define EOF_PENDING	2
#define INHERITED	4

struct opt_entry {
	struct opt_entry *optlink;
	unsigned opt_dev_no;
	unsigned opt_save;
	} ;

struct open_entry {
	struct open_entry *oplink;
	unsigned open_pid;
	unsigned oflags;
	struct msg_hdr ohdr;
	} ;

struct output_buffer {
	struct output_buffer *olink;
	struct msg_hdr obuf;
	char ob_buf[80];
	} ;

struct abort_msg {
	char mtype;
	char my_tty;
	unsigned exc_bits;
	} ;

#define STTY_SET_PARAMETERS		1
#define STTY_QUERY_PARAMETERS	2
#define STTY_RESET_PARAMETERS	3
#define STTY_SEND_BREAK			4
#define STTY_SET_OPTIONS		5
#define STTY_GET_OPTIONS		6

struct stty_entry {	/* message format */
	char 		mtype;			/* STTY_MESSAGE */
	char		stty_devno;
	char		stty_type;
	unsigned	stty_baud;		/* 0 .. 64K */
	char		stty_parity;	/* 0 - NONE, 1 - ODD, 3 - EVEN , 5 - MARK, 7 - SPACE */
	char		stty_stop_bits;	/* (1 .. 2) * 2 */
	char		stty_data_bits;	/* 5 .. 8 */
	} ;

struct adopt_device_msg {
	char mtype;
	char mdevno;
	char mdev_name[6];
	} ;

struct query_device_msg {
	char mtype;
	char mdevno;
	char mdev_name[6];
	unsigned mdev_tid;
	} ;

struct new_device_entry {
	char new_dev_name[6];
	unsigned new_dev_tid;
	} ;


/*
 * Device Open Error types (top bit will be set)
 */

#define BAD_DEVICE			1
#define BAD_MODE			2
#define NO_BUF_SPACE		3
#define DEVICE_BUSY			4

/*
 *	definition of device types which are recognized by QNX
 */

#define DEV_COLOUR_NW	1		/* Colour, no horizontal wait required */
#define DEV_COLOUR		2		/* Colour display */
#define DEV_MONOCHROME	3		/* Monochrome display */
#define DEV_ASCII		4		/* dumb ASCII terminal (8250 controller) */
#define DEV_PARALLEL	5		/* Parallel printer */
