/*
 * The following TIDs are reserved for the system tasks.
 * They MUST NOT be changed once the system has been released.
 */

#define TASK_ADMIN		1
#define FSYS_ADMIN		2
#define DEV_ADMIN		3
#define IDLE			4
#define NET_ADMIN		5
#define NUM_BOOT_TASKS	5
#define NUM_SYS_TASKS	5


/*
 * Task States
 */

#define DEAD			0
#define READY			1
#define SEND_BLOCKED	2
#define RECEIVE_BLOCKED	3
#define REPLY_BLOCKED	4
#define HELD			5
#define NET_BLOCKED		6


/*
 * Task flag bits
 */

#define TO_BE_HELD		0x0001
#define DOOMED			0x0002
#define ESCAPED_SHELL	0x0004
#define FETAL_SHELL		0x0008
#define CONCURRENT		0x0010
#define ADMIN			0x0020
#define FIXED			0x0040	/* Shared by code flags */
#define PRIVILEGED		0x0080	/* Shared by code flags */
#define INFORM			0x0100
#define TRANSFORMER		0x0200
#define SUPRESS_STDIN	0x0400
#define WAIT			0x2000
#define PART_NDP_SAVE	0x4000
#define FULL_NDP_SAVE	0x8000

/*
 * Code record flag bits
 */

#define NEEDS_NDP		0x01
#define SHAREABLE		0x02
#define SET_OWNER		0x04
#define LOAD_HIGH		0x08
#define REMOTE_OK		0x10


/*
 * System flags in sflags field
 */

#define NDP_INSTALLED	0x01
#define PASSWORDS_ON	0x02
#define SYSTEM_UP		0x04
#define REMOTE_CREATES	0x08


/*
 * Account flag bits
 */

#define MAIL_WAITING	0x01		/* cleared when read */


/*
 * System port associations for hardware events
 */

#define CLOCK_EVENT		1
#define TTY_EVENT		2
#define DISK_EVENT		7
#define PRINTER_EVENT	8



/*
 * System exception bits
 */

#define EXC_HANG_UP			0x0001
#define EXC_BREAK			0x0002
#define EXC_QUIT			0x0004
#define EXC_ILLEGAL_INSTR	0x0008
#define EXC_COMM_ERROR		0x0010
#define EXC_MISSING_SLIB	0x0040
#define EXC_FLOAT			0x0080
#define EXC_KILL			0x0100
#define EXC_MEM_VIOLATION	0x0400
#define EXC_PRIV_VIOLATION	0x0800
#define EXC_ALARM_CLOCK		0x2000
#define EXC_TERM			0x4000
#define EXC_DIV_ZERO		0x8000


/*
 Kernal constants
 */

#define PRIO_BITS			4
#define NUM_PRIORITIES		(1 << PRIO_BITS)
#define PRIO_MASK			(NUM_PRIORITIES - 1)
#define MIN_USER_PRIORITY	PRIO_MASK
#define DEFLT_USER_PRIORITY	8
#define MAX_USER_PRIORITY	4
#define SYS_PRIORITY		1
#define BITS_PER_BLK		4
#define BLK_SIZE			(1<<BITS_PER_BLK)	/* For the 8086/8088 */
#define ONE_HUNDRED_MSEC	2
#define FIFTY_MSEC			1

#define TID_BITS		8
#define TID_MASK		((1 << TID_BITS) - 1)
#define VID_FLAG		0x8000

#define SUPER_USER		255
#define SOH_HDR_SIZE	(14-1)
#define INIT_ALLOC_SIZE	840		/* room for one file, and three ttys */
#define MIN_STACK_SIZE	1000
#define INT_STACK_SIZE	64
#define OS_SEG			0x60
#define MAGIC_MEM_SIZE	0x80

#define enable	asm("sti");
#define disable asm("cli");

struct system_info {			/* Note: This struct overlays task_table[0]	*/
	unsigned task_base;			/* 00 */
	char num_tasks;				/* 02 */
	char num_events;			/* 03 */
	char num_priorities;		/* 04 */
	char num_ttys;				/* 05 */
	unsigned num_mblks;			/* 06 */
	unsigned sflags;			/* 08 */
	char slice_size;			/* 0a */
	char num_vtasks;			/* 0b */
	unsigned version_number		/* 0c */
	unsigned node_number;		/* 0e */
	unsigned num_nodes;			/* 10 */
	char num_tasks_free;		/* 12 */
	char num_events_free;		/* 13 */
	char sys_info_spare;		/* 14 */
	char num_vtasks_free;		/* 15 */
	unsigned num_mblks_free;	/* 16 */
	char num_pkts;				/* 18 */
	char num_pkts_free;			/* 19 */
	unsigned server_nid;		/* 1a */
	unsigned server_tid;		/* 1c */
	char num_names;				/* 1e */
	} ;
/* 31 */	

struct task_entry {
	struct task_entry *tlink;	/* 00 */
	unsigned tid;				/* 02 */
	unsigned blocked_on;		/* 04 */
	char *send_msg;				/* 06 */
	unsigned msg_length;		/* 08 */

	unsigned data_seg;			/* 0a */
	unsigned data_size;			/* 0c */
	char state;					/* 0e */
	char priority;				/* 0f */
	char user_num;				/* 10 */
	char task_tty;				/* 11 */
	char group_num;				/* 12 */
	char code_index;			/* 13 */ /* into table at 46d1? */
/* 20 */
	unsigned tflags;			/* 14 */
	struct task_entry *sender_queue;		/* 16 */
	char *reply_msg;			/* 18 */
	struct sys_mem_entry *segment_list;		/* 1a */
	struct task_entry *father;	/* 1c */

	struct task_entry *son;		/* 1e */
	struct task_entry *brother;	/* 20 */
	unsigned task_tty_nid;		/* 22 */
	unsigned cd_disk_id;		/* 24 */
	unsigned cd_nid;			/* 26 */
/* 40 */
	unsigned cd_block;			/* 28 */
	unsigned cd_index;			/* 2a */
	unsigned ext_cd_block;		/* 2c */
	char cd_drive;				/* 2e */
	char team_num;				/* 2f */
	} ;
/* 48 */


struct vtask_entry {
	struct task_entry *tlink;	/* 00 */
	unsigned tid;				/* 02 */
	unsigned blocked_on;		/* 04 */
	char *send_msg;				/* 06 */
	unsigned msg_length;		/* 08 */
	unsigned data_seg;			/* 0a */
	unsigned data_size;			/* 0c */
	char state;					/* 0e */
	char priority;				/* 0f */
	char user_num;				/* 10 */
	char task_tty;				/* 11 */
	char group_num;				/* 12 */
	char code_index;			/* 13 */
	unsigned tflags;			/* 14 */
	struct task_entry *sender_queue;/* 16 */
	char *reply_msg;			/* 18 */
	unsigned remote_nid;		/* 1a */
	unsigned remote_tid;		/* 1c */
	unsigned remote_vid;		/* 1e */
	unsigned local_tid;			/* 20 */
	unsigned task_tty_nid;		/* 22 */
	unsigned cd_disk_id;		/* 24 */
	unsigned cd_nid;			/* 26 */
	unsigned cd_block;			/* 28 */
	unsigned cd_index;			/* 2a */
	unsigned ext_cd_block;		/* 2c */
	char cd_drive;				/* 2e */
	char slice_cnt;				/* 2f */
	} ;



struct event_entry {
	struct task_entry *tlink;
	unsigned tid;
	unsigned blocked_on;
	char *send_msg;
	unsigned msg_length;
	unsigned data_seg;
	unsigned data_size;		/* not currently used */
	char state;
	unsigned event_cnt;
	} ;
/* 17 bytes */

struct net_queue_entry {
	struct net_queue_entry *nlink;
	char net_queue_class;
	char net_queue_type;
	unsigned target_nid;
	unsigned target_vid;
	unsigned source_tid;
	unsigned target_args[2];
	} ;
/*  8 bytes */

struct code_entry {
	unsigned code_seg;		/* 00 */
	char link_count;		/* 02 */
	char code_owner;		/* 03 */
	char code_group;		/* 04 */
	char code_flags;		/* 05 */
	unsigned code_size;		/* 06 (in paragraphs?) */
	unsigned init_data_size;/* 08 */
	unsigned const_size;	/* 0a */
	unsigned const_chksum;	/* 0c */
	unsigned stack_size;	/* 0e */
	unsigned code_spare[2];	/* 10 */
	char code_name[0];		/* 14 */
	} ;
/* 14 + len(name) */

struct account_entry {			/* This is kept in an extra segment */
	char account_links;			/* Number of task groups using this entry */
	unsigned account_flags;
	char account_tty;
	unsigned account_nid;
	char account_group_num;
	char account_user_num;
	unsigned account_login_time[2];
	char account_name[17];		/* 16 characters plus the terminating null */
	char account_login_dir[31];
	unsigned account_spare[10];
	};


struct name_entry {
	unsigned name_nid;
	unsigned name_tid;
	unsigned name_dat;
	char name_name[9];
	} ;


struct sys_mem_entry {
	struct sys_mem_entry *mlink;
	unsigned blk_no;
	unsigned nblks;
	} ;


struct user_memory {	/* cs:00-7f */	/* magic memory */
	unsigned User_sp;			/* 00 */
	unsigned User_ss;			/* 02 */
	unsigned Exc_bits[2];		/* 04 */
	unsigned Exc_allow[2];		/* 08 */
	unsigned Exc_pending[2];	/* 0c */
	unsigned Exc_fp;			/* 10 */
	unsigned Exc_pc;			/* 12 */
	unsigned Exc_permit[2];		/* 14 */
	unsigned Exit_status;		/* 18 */
	unsigned Error_status;		/* 1a */
	unsigned My_tid;			/* 1c */
	unsigned Dads_tid;			/* 1e */
	char Break_count;			/* 20 */
	char My_tty;				/* 21 */
	unsigned Stack_base;		/* 22 */
	char Cmd_flags;				/* 24 */
	char My_priority;			/* 25 */
	unsigned Stdin;				/* 26 */
	unsigned Stdout;			/* 28 */
	unsigned Stderr;			/* 2a */
    unsigned Free_list;			/* 2c */
	unsigned End_of_free;		/* 2e */
	unsigned File_list;			/* 30 */
	unsigned Argc;				/* 32 */
	unsigned Argv;				/* 34 */
	unsigned Save;				/* 36 */
	unsigned Temp[10];			/* 38 */
	unsigned Reserved[10];		/* 4C */
	char Code_flags;			/* 60 */
	char User_num;				/* 61 */
	char Group_num;				/* 62 */
	char Team_num;				/* 63 */
	char char_spare;			/* 64 */
	unsigned My_nid;			/* 65 */
	unsigned Dads_nid;			/* 67 */
	unsigned My_tty_nid;		/* 69 */
	unsigned My_cd_nid;			/* 6B */
	char Release_num;			/* 6C */
	} ;
